# remirepo spec file for php-pecl-skywalking
#
# Copyright (c) 2019-2022 Remi Collet
# License: CC-BY-SA
# http://creativecommons.org/licenses/by-sa/4.0/
#
# Please, preserve the changelog entries
#
%if 0%{?scl:1}
%scl_package       php-pecl-skywalking
%else
%global _root_bindir %{_bindir}
%endif

%global pecl_name  skywalking
# not supported by upstream
%global with_zts   0
%global ini_name   40-%{pecl_name}.ini
%global sources    %{pecl_name}-%{version}
%global _configure ../%{sources}/configure

Summary:        The PHP instrument agent for Apache SkyWalking
Name:           %{?scl_prefix}php-pecl-%{pecl_name}
Version:        5.0.1
Release:        3%{?dist}%{!?nophptag:%(%{__php} -r 'echo ".".PHP_MAJOR_VERSION.".".PHP_MINOR_VERSION;')}
License:        Apache-2.0
URL:            https://pecl.php.net/package/%{pecl_name}

Source0:        https://pecl.php.net/get/%{sources}.tgz
# awfull hack, use a bundled rust registry
Source1:        makedeps.sh
Source2:        https://pecl.php.net/get/%{pecl_name}-deps-%{version}.tgz

%if 0%{?rhel} == 7 && 0%{?dtsversion} == 0
BuildRequires:  devtoolset-6-toolchain
%global dtsversion 6
%global dtsenable source /opt/rh/devtoolset-6/enable
%global dtsprefix devtoolset-6-
%endif
BuildRequires:  make
BuildRequires:  %{?dtsprefix}gcc
BuildRequires:  %{?scl_prefix}php-devel >= 7.0
BuildRequires:  %{?scl_prefix}php-pear
BuildRequires:  %{?scl_prefix}php-curl
BuildRequires:  %{?scl_prefix}php-json
BuildRequires:  %{?scl_prefix}php-pcre
BuildRequires:  libcurl-devel
BuildRequires:  cargo >= 1.52
BuildRequires:  rustfmt
%if 0%{?fedora} >= 27 || 0%{?rhel} >= 9
BuildRequires:  rust-packaging
%endif
# Using system rust registry
#BuildRequires:  (crate(anyhow/default) >= 1.0.57 with crate(anyhow/default) < 2.0.0~)
#BuildRequires:  (crate(cbindgen/default) >= 0.23.0 with crate(cbindgen/default) < 0.24.0~)
#BuildRequires:  (crate(futures-util/default) >= 0.3.21 with crate(futures-util/default) < 0.4.0~)
#BuildRequires:  (crate(gethostname/default) >= 0.2.3 with crate(gethostname/default) < 0.3.0~)
#*************:  (crate(ipc-channel/default) >= 0.16.0 with crate(ipc-channel/default) < 0.17.0~)
#*************:  (crate(local-ip-address/default) >= 0.4.4 with crate(local-ip-address/default) < 0.5.0~)
#BuildRequires:  (crate(log/default) >= 0.4.1 with crate(log/default) < 0.5.0~)
#BuildRequires:  (crate(prost-derive/default) >= 0.8.0 with crate(prost-derive/default) < 0.9.0~)
#BuildRequires:  (crate(prost/default) >= 0.8.0 with crate(prost/default) < 0.9.0~)
#BuildRequires:  (crate(rand/default) >= 0.8.5 with crate(rand/default) < 0.9.0~)
#BuildRequires:  (crate(serde/default) >= 1.0.0 with crate(serde/default) < 2.0.0~)
#BuildRequires:  (crate(serde/serde_derive) >= 1.0.0 with crate(serde/serde_derive) < 2.0.0~)
#BuildRequires:  (crate(serde_json/default) >= 1.0.0 with crate(serde_json/default) < 2.0.0~)
#BuildRequires:  (crate(simplelog/default) >= 0.12.0 with crate(simplelog/default) < 0.13.0~)
#BuildRequires:  (crate(tokio/default) >= 1.0.0 with crate(tokio/default) < 2.0.0~)
#BuildRequires:  (crate(tokio/full) >= 1.0.0 with crate(tokio/full) < 2.0.0~)
#*************:  (crate(tonic-build/default) >= 0.5.2 with crate(tonic-build/default) < 0.6.0~)
#*************:  (crate(tonic/default) >= 0.5.2 with crate(tonic/default) < 0.6.0~)
#BuildRequires:  (crate(uuid/default) >= 1.0.0 with crate(uuid/default) < 2.0.0~)
#BuildRequires:  (crate(uuid/v4) >= 1.0.0 with crate(uuid/v4) < 2.0.0~)

Requires:       %{?scl_prefix}php(zend-abi) = %{php_zend_api}
Requires:       %{?scl_prefix}php(api) = %{php_core_api}
Requires:       %{?scl_prefix}php-curl
Requires:       %{?scl_prefix}php-json
Requires:       %{?scl_prefix}php-pcre

Provides:       %{?scl_prefix}php-%{pecl_name}               = %{version}
Provides:       %{?scl_prefix}php-%{pecl_name}%{?_isa}       = %{version}
Provides:       %{?scl_prefix}php-pecl(%{pecl_name})         = %{version}
Provides:       %{?scl_prefix}php-pecl(%{pecl_name})%{?_isa} = %{version}


%description
The package is the PHP instrumentation agent, which is compatible with
Apache SkyWalking backend and others compatible agents/SDKs.

Package built for PHP %(%{__php} -r 'echo PHP_MAJOR_VERSION.".".PHP_MINOR_VERSION;')%{?scl: as Software Collection (%{scl} by %{?scl_vendor}%{!?scl_vendor:rh})}.


%prep
%setup -q -c -a2

# Don't install/register tests
sed -e 's/role="test"/role="src"/' \
    %{?_licensedir:-e '/LICENSE/s/role="doc"/role="src"/' } \
    -i package.xml

pushd %{sources}
# Create cargo configuration to use vendor directory
mkdir .cargo
cat << EOF | tee .cargo/config.toml
[build]
rustc = "%{_root_bindir}/rustc"

[env]
CFLAGS = "%{build_cflags}"
CXXFLAGS = "%{build_cxxflags}"
LDFLAGS = "%{build_ldflags}"

[term]
verbose = true

[source.crates-io]
replace-with = "vendored-sources"

[source.vendored-sources]
directory = "$(dirname $PWD)/mycargo"
EOF

: Check version
extver=$(sed -n '/#define PHP_SKYWALKING_VERSION /{s/.* "//;s/".*$//;p}' php_skywalking.h)
if test "x${extver}" != "x%{version}%{?prever}"; then
   : Error: Upstream version is ${extver}, expecting %{version}%{?prever}.
   exit 1
fi
popd

mkdir NTS

%if %{with_zts}
mkdir ZTS
%endif

# Create configuration file
cat > %{ini_name} << 'EOF'
; Enable %{pecl_name} extension module
extension=%{pecl_name}.so

; Configuration
;skywalking.enable = Off
;skywalking.service = 'hello_skywalking'
;skywalking.service_instance = ''
;skywalking.oap_version = '9.0.0'
;skywalking.oap_cross_process_protocol = '3.0'
;skywalking.oap_authentication = ''
;skywalking.grpc_address = '127.0.0.1:11800'
;skywalking.grpc_tls_enable = Off
;skywalking.grpc_tls_pem_root_certs = ''
;skywalking.grpc_tls_pem_private_key = ''
;skywalking.grpc_tls_pem_cert_chain = ''
;skywalking.log_level = 'disable'
;skywalking.log_path = '/tmp/skywalking-php.log';
;skywalking.curl_response_enable = Off
;skywalking.error_handler_enable = Off
;skywalking.mq_max_message_length = 20480
;skywalking.mq_unique = 0
;skywalking.sample_n_per_3_secs = -1

EOF

: Required rust version
grep -h rust-version mycargo/*/Cargo.toml NTS/Cargo.toml | sort -u | tail -n 4


%build
%{?dtsenable}

export CARGO_HOME=$PWD/mycargo

cd %{sources}
%{__phpize}

cd ../NTS
cp -r ../%{sources}/.cargo .cargo
%configure \
    --enable-skywalking \
    --with-php-config=%{__phpconfig}
make %{?_smp_mflags}

%if %{with_zts}
cd ../ZTS
cp -r ../%{sources}/.cargo .cargo
%configure \
    --enable-skywalking \
    --with-php-config=%{__ztsphpconfig}
make %{?_smp_mflags}
%endif


%install
%{?dtsenable}

make -C NTS install INSTALL_ROOT=%{buildroot}

# install config file
install -D -m 644 %{ini_name} %{buildroot}%{php_inidir}/%{ini_name}

# Install XML package description
install -D -m 644 package.xml %{buildroot}%{pecl_xmldir}/%{name}.xml

%if %{with_zts}
make -C ZTS install INSTALL_ROOT=%{buildroot}

install -D -m 644 %{ini_name} %{buildroot}%{php_ztsinidir}/%{ini_name}
%endif

# Documentation
cd %{sources}
for i in $(grep 'role="doc"' ../package.xml | sed -e 's/^.*name="//;s/".*$//')
do install -Dpm 644 $i %{buildroot}%{pecl_docdir}/%{pecl_name}/$i
done
%{!?_licensedir:install -Dpm 644 LICENSE %{buildroot}%{pecl_docdir}/%{pecl_name}/LICENSE}


%if 0%{?fedora} < 24 && 0%{?rhel} < 8
# when pear installed alone, after us
%triggerin -- %{?scl_prefix}php-pear
if [ -x %{__pecl} ] ; then
    %{pecl_install} %{pecl_xmldir}/%{name}.xml >/dev/null || :
fi

# posttrans as pear can be installed after us
%posttrans
if [ -x %{__pecl} ] ; then
    %{pecl_install} %{pecl_xmldir}/%{name}.xml >/dev/null || :
fi

%postun
if [ $1 -eq 0 -a -x %{__pecl} ] ; then
    %{pecl_uninstall} %{pecl_name} >/dev/null || :
fi
%endif


%check
OPT="-n -d extension=curl.so -d extension=json.so"

# Minimal load test for NTS extension
%{__php} $OPT \
    --define extension=$PWD/NTS/modules/%{pecl_name}.so \
    --modules | grep %{pecl_name}

%if %{with_zts}
# Minimal load test for ZTS extension
%{__ztsphp} $OPT \
    --define extension=$PWD/ZTS/modules/%{pecl_name}.so \
    --modules | grep %{pecl_name}
%endif


%files
%{?_licensedir:%license %{sources}/LICENSE}
%doc %{pecl_docdir}/%{pecl_name}
%{pecl_xmldir}/%{name}.xml

%config(noreplace) %{php_inidir}/%{ini_name}
%{php_extdir}/%{pecl_name}.so

%if %{with_zts}
%config(noreplace) %{php_ztsinidir}/%{ini_name}
%{php_ztsextdir}/%{pecl_name}.so
%endif


%changelog
* Mon Sep  4 2023 Remi Collet <remi@remirepo.net> - 5.0.1-3
- build out of sources tree

* Fri Jul  1 2022 Remi Collet <remi@remirepo.net> - 5.0.1-2
- use cargo vendor feature to reduce sources size

* Thu Jun 30 2022 Remi Collet <remi@remirepo.net> - 5.0.1-1
- update to 5.0.1
- use bundled rust registry

* Mon Jun 27 2022 Remi Collet <remi@remirepo.net> - 5.0.0-1
- update to 5.0.0
- open https://github.com/SkyAPM/SkyAPM-php-sdk/issues/493
  Impossible to build offline

* Wed Nov  3 2021 Remi Collet <remi@remirepo.net> - 4.2.0-1
- update to 4.2.0

* Thu Aug 26 2021 Remi Collet <remi@remirepo.net> - 4.1.3-1
- update to 4.1.3
- open https://github.com/SkyAPM/SkyAPM-php-sdk/issues/429
  impossible to build with shared grpc library

* Fri Apr 30 2021 Remi Collet <remi@remirepo.net> - 4.1.2-1
- update to 4.1.2

* Thu Nov 19 2020 Remi Collet <remi@remirepo.net> - 4.1.1-1
- update to 4.1.1
- drop patch merged upstream

* Fri Nov 13 2020 Remi Collet <remi@remirepo.net> - 4.1.0-1
- update to 4.1.0
- drop patches merged upstream
- add patch to fix build with PHP 8 from
  https://github.com/SkyAPM/SkyAPM-php-sdk/pull/294

* Thu Oct 15 2020 Remi Collet <remi@remirepo.net> - 4.0.0-2
- add patch to properly check available command from
  https://github.com/SkyAPM/SkyAPM-php-sdk/pull/275
- add patch to fix build with PHP 8 from
  https://github.com/SkyAPM/SkyAPM-php-sdk/pull/276

* Thu Oct 15 2020 Remi Collet <remi@remirepo.net> - 4.0.0-1
- update to 4.0.0
- add patch to properly use cxx for link from
  https://github.com/SkyAPM/SkyAPM-php-sdk/pull/274

* Fri Jun  5 2020 Remi Collet <remi@remirepo.net> - 3.3.2-1
- update to 3.3.2

* Wed Jun  3 2020 Remi Collet <remi@remirepo.net> - 3.3.1-1
- update to 3.3.1

* Wed May 13 2020 Remi Collet <remi@remirepo.net> - 3.3.0-1
- update to 3.3.0

* Mon Mar 23 2020 Remi Collet <remi@remirepo.net> - 3.2.8-1
- update to 3.2.8

* Thu Feb 13 2020 Remi Collet <remi@remirepo.net> - 3.2.6-1
- update to 3.2.6

* Tue Feb 11 2020 Remi Collet <remi@remirepo.net> - 3.2.5-1
- update to 3.2.5

* Mon Dec  9 2019 Remi Collet <remi@remirepo.net> - 3.2.4-1
- initial package
